#cimport csamtools

from pysam.csamtools cimport *

import collections, array, struct, sys
import CGAT.Experiment as E

FLAGS = {
    1: 'paired',
    2: 'proper_pair',
    4: 'unmapped',
    8: 'mate_unmapped',
    16: 'reverse',
    32: 'mate_reverse',
    64: 'read1',
    128: 'read2',
    256: 'secondary',
    512: 'qc_fail',
    1024: 'duplicate'  }

cdef struct CountsType:
    int is_unmapped
    int mate_is_unmapped
    int is_paired
    int is_read1
    int is_read2
    int is_proper_pair
    int is_qcfail
    int is_duplicate

def count( Samfile samfile,
           remove_rna,
           rna,
           filename_fastq = None,
           outfile_details = None):
    '''

    '''
    cdef bint _remove_rna = remove_rna

    cdef AlignedRead read

    # counters
    cdef int ninput = 0
    cdef int nduplicates = 0
    # number of reads overlapping RNA (if rna != None)
    cdef int nrna = 0
    # number of reads not overlap RNA (if rna != None)
    cdef int n_norna = 0
    # number of reads present after filtering
    cdef int nfiltered = 0

    cdef int max_hi = 0
    # count nh, nm tags
    nh_filtered, nm_filtered = collections.defaultdict( int ), collections.defaultdict( int )
    nh_all, nm_all = collections.defaultdict( int ), collections.defaultdict( int )
    mapq_filtered, mapq_all = collections.defaultdict( int ), collections.defaultdict( int )

    cdef int * flags_counts = <int*>calloc( len(FLAGS), sizeof(int) )

    # helper variables
    cdef int last_tid = -1
    cdef int last_pos = 0

    duplicates = collections.defaultdict( int )
    counts = collections.defaultdict( int )
    cdef int count = 0

    cdef int tid = -1
    cdef int flag

    cdef uint8_t * v
    cdef int32_t nm
    cdef int32_t nh
    cdef int32_t hi
    cdef int x
    cdef int lflags = len(FLAGS)
    cdef int f

    # detailed counting
    cdef FastqProxy fq
    cdef int64_t index, fastq_nreads
    cdef CountsType * fastq_counts
    cdef CountsType * fastq_count
    cdef char * read_name
    cdef int count_fastq = filename_fastq != None

    if count_fastq:
        E.info( "reading fastq file" )
        # Using a python dictionary here is due
        # for a large amount of memory usage.
        # Alternatives to dictionary
        # 1. POSIX hash tables (hsearch,...) or trees: very slow
        # 2. custom hash implementation: worth the effort?
        # 3. Sorted list and binary search: too slow for many lookups
        reads = {}
        fastqfile = Fastqfile( filename_fastq )
        fastq_nreads = 0
        for fq in fastqfile:
            reads[fq.name] = fastq_nreads
            fastq_nreads += 1
        
        E.info( "read %i reads" % fastq_nreads )

        E.info( "allocating %i bytes" % (fastq_nreads * sizeof(CountsType) ) )

        fastq_counts = <CountsType *>calloc( fastq_nreads, sizeof( CountsType ) )
        if fastq_counts == NULL:
            raise ValueError( "could not allocated memory: %i bytes" % (fastq_nreads * sizeof(CountsType) ))
 
    for read in samfile:

        if count_fastq:
            try:
                fastq_count = &fastq_counts[ reads[read.qname] ]
            except KeyError:
                continue
        
            if read.is_unmapped: fastq_count.is_unmapped += 1
            if read.mate_is_unmapped: fastq_count.mate_is_unmapped += 1
            if read.is_paired: fastq_count.is_paired += 1
            if read.is_read1: fastq_count.is_read1 += 1
            if read.is_read2: fastq_count.is_read2 += 1
            if read.is_proper_pair: fastq_count.is_proper_pair += 1
            if read.is_qcfail: fastq_count.is_qcfail += 1
            if read.is_duplicate: fastq_count.is_duplicate += 1
 
        flag = read._delegate.core.flag

        # is paired and read2
        #if flag & 1 and flag & 128:
            # ignore second reads to avoid double counting pairs
            # this needs to be done properly by counting in
            # pairs
        #    continue

        ninput += 1

        f = 1
        for x from 0 <= x < lflags:
            if flag & f: flags_counts[x] += 1
            f = f << 1

        # get maximum NI field
        v = bam_aux_get(read._delegate, 'HI')
        if v != NULL:
            hi = <int32_t>bam_aux2i(v)
            if hi > max_hi: max_hi = hi

        v = bam_aux_get(read._delegate, 'NH')
        if v != NULL:
            nh = <int32_t>bam_aux2i(v)
            nh_all[nh] += 1
        else:
            nh = -1

        v = bam_aux_get(read._delegate, 'NM')
        if v != NULL:
            nm = <int32_t>bam_aux2i(v)
            nm_all[nm] += 1
        else:
            nm = -1

        mapq_all[read.mapq] += 1

        # skip unmapped reads
        if read._delegate.core.flag & 4: continue

        if read.tid != last_tid:
            contig = samfile.getrname( read.rname )

        # note: does not take into account gaps within reads
        # or partial overlap.
        if rna:
            if rna.contains( contig, read.pos, read.pos + read.alen ):
                nrna += 1
                if _remove_rna: continue
            else:
                n_norna += 1

        nfiltered += 1

        if nh >= 0: nh_filtered[nh] += 1
        if nm >= 0: nm_filtered[nm] += 1
        mapq_filtered[read.mapq] += 1

        # duplicate analysis - simply count per start position
        # ignoring sequence and strand
        if read.tid == last_tid and read.pos == last_pos:
            count += 1
            nduplicates += 1
            continue

        if count > 1:
            counts[count] += 1

        count = 1
        last_tid, last_pos = read.tid, read.pos

    E.info( "finished computing counts" )

    counter = E.Counter()
    
    counter.input = ninput
    counter.filtered = nfiltered
    counter.rna = nrna
    counter.no_rna = n_norna
    counter.duplicates = nduplicates

    # convert flags to labels
    t = {}
    f = 1
    for x from 0 <= x < lflags:
        t[FLAGS[f]] = flags_counts[x]
        f = f << 1

    # count based on fastq data
    cdef int total_pairs = 0
    cdef int total_pair_is_unmapped = 0
    cdef int total_pair_is_proper_uniq = 0
    cdef int total_pair_is_proper_mmap = 0
    cdef int total_pair_is_proper_duplicate = 0
    cdef int total_pair_not_proper_uniq = 0
    cdef int total_pair_is_incomplete = 0
    cdef int total_pair_is_other = 0

    if count_fastq:
        E.info( "aggregating counts" )
        for qname, index in reads.items():
            fastq_count = &fastq_counts[index]

            # paired read counting
            if fastq_count.is_paired : 
                total_pairs += 1

                if fastq_count.is_unmapped == 2:
                    # an unmapped read pair
                    total_pair_is_unmapped += 1
                elif fastq_count.is_proper_pair == 2: 
                    # a unique proper pair
                    total_pair_is_proper_uniq +=1
                    # a duplicate unique proper pair
                    if fastq_count.is_duplicate == 2:
                        total_pair_is_proper_duplicate +=1
                elif fastq_count.is_proper_pair > 2:
                    # proper pairs that map to mulitple locations
                    total_pair_is_proper_mmap +=1
                elif fastq_count.is_read1 == 1 \
                        and fastq_count.is_read2 == 1 \
                        and fastq_count.is_proper_pair == 0:
                    # pair which map each read once, but not uniquely
                    total_pair_not_proper_uniq += 1
                elif (fastq_count.is_read1 == 1 and fastq_count.is_read2 == 0) \
                        or (fastq_count.is_read1 == 0 and fastq_count.is_read2 == 1):
                    # an incomplete pair - one read of a pair matches uniquel, but not the other
                    total_pair_is_incomplete += 1
                else:
                    total_pair_is_other += 1

        counter.total_pairs = total_pairs
        counter.total_pair_is_unmapped = total_pair_is_unmapped
        counter.total_pair_is_proper_uniq = total_pair_is_proper_uniq
        counter.total_pair_is_incomplete = total_pair_is_incomplete
        counter.total_pair_is_proper_duplicate = total_pair_is_proper_duplicate
        counter.total_pair_is_proper_mmap = total_pair_is_proper_mmap
        counter.total_pair_not_proper_uniq = total_pair_not_proper_uniq
        counter.total_pair_is_other = total_pair_is_other

        if outfile_details:
            if outfile_details != sys.stdout:
                # later: get access FILE * object
                outfile_details.write( "read\tis_unmapped\tmate_is_unmapped\tis_paired\tis_read1\tis_read2\tis_proper_pair\tis_qcfail\tis_duplicate\n" )
                for qname, index in reads.items():
                    fastq_count = &fastq_counts[index]

                    outfile_details.write( "%s\t%s\n" % (qname, "\t".join( \
                                map(str,
                                    (fastq_count.is_unmapped,
                                     fastq_count.mate_is_unmapped,
                                     fastq_count.is_paired,
                                     fastq_count.is_read1,
                                     fastq_count.is_read2,
                                     fastq_count.is_proper_pair,
                                     fastq_count.is_qcfail,
                                     fastq_count.is_duplicate
                                    ) )) ))

            else:
                # output to stdout much quicker
                printf("read\tis_unmapped\tmate_is_unmapped\tis_paired\tis_read1\tis_read2\tis_proper_pair\tis_qcfail\tis_duplicate\n" )
                for qname, index in reads.items():
                    fastq_count = &fastq_counts[index]
                    read_name = qname
                    printf( "%s\t%i\t%i\t%i\t%i\t%i\t%i\t%i\t%i\n",
                            read_name,
                            fastq_count.is_unmapped,
                            fastq_count.mate_is_unmapped,
                            fastq_count.is_paired,
                            fastq_count.is_read1,
                            fastq_count.is_read2,
                            fastq_count.is_proper_pair,
                            fastq_count.is_qcfail,
                            fastq_count.is_duplicate )


    return counter, t, nh_filtered, nh_all, nm_filtered, nm_all, mapq_filtered, mapq_all, max_hi



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
CGAT.py - CGAT project specific functions
=========================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

The :mod:`CGAT` module contains various utility functions
for working on CGAT projects.

API
----
'''

from ruffus import *

import os, sys, re, shutil
import Experiment as E

PROJECT_ROOT = '/ifs/projects'

def getProjectId():
    '''cgat specific method: get the (obfuscated) project id
    based on the current working directory.
    '''
    curdir = os.path.abspath(os.getcwd())
    if not curdir.startswith( PROJECT_ROOT ):
        raise ValueError( "method getProjectId no called within %s" % PROJECT_ROOT )
    prefixes = len(PROJECT_ROOT.split("/"))
    # patch for projects in mini directory
    if "/mini/" in curdir: prefixes += 1
    rootdir = "/" + os.path.join( *(curdir.split( "/" )[:prefixes+1]) )
    f = os.path.join( rootdir, "sftp", "web" )
    if not os.path.exists(f):
        raise OSError( "web directory at '%s' does not exist" % f )
    target = os.readlink( f )
    return os.path.basename( target )


def getProjectName():
    '''cgat specific method: get the name of the project 
    based on the current working directory.'''

    curdir = os.path.abspath(os.getcwd())
    if not curdir.startswith( PROJECT_ROOT ):
        raise ValueError( "method getProjectName no called within %s" % PROJECT_ROOT )
    prefixes = len(PROJECT_ROOT.split("/"))
    return curdir.split( "/" )[prefixes]

def publish_report( prefix = "", 
                    patterns = [], 
                    project_id = None,
                    prefix_project = "/ifs/projects",
                    export_files = None,
                    ):
    '''publish report into web directory.

    Links export directory into web directory.

    Copies html pages and fudges links to the pages in the
    export directory.

    If *prefix* is given, the directories will start with prefix.

    *patterns* is an optional list of two-element tuples (<pattern>, replacement_string).
    Each substitutions will be applied on each file ending in .html.

    If *project_id* is not given, it will be looked up. This requires
    that this method is called within a subdirectory of PROJECT_ROOT.

    *export_files* is a dictionary of files to be exported. The key
    of the dictionary denotes the targetdirectory within the web
    directory. The values in the dictionary are the files to be
    linked to in the direcotry. For example::

        exportfiles = {
            "bamfiles" : glob.glob( "*/*.bam" ) + glob.glob( "*/*.bam.bai" ),
            "bigwigfiles" : glob.glob( "*/*.bw" ),
            }

    .. note::
       This function is CGAT specific.

    '''

    if not prefix:
        prefix = PARAMS.get( "report_prefix", "" )

    web_dir = PARAMS["web_dir"]
    if project_id == None: 
        project_id = getProjectId()

    src_export = os.path.abspath( "export" )
    dest_report = prefix + "report"
    dest_export = prefix + "export"

    curdir = os.path.abspath( os.getcwd() )

    def _link( src, dest ):
        '''create links. 

        Only link to existing targets.
        '''
        dest = os.path.abspath( os.path.join( PARAMS["web_dir"], dest ) )
        if os.path.exists( dest ):
            os.remove(dest)
        
        if os.path.exists( src ):
            #IMS: check if base path of dest exists. This allows for prefix to be a 
            #nested path structure e.g. project_id/
            if not os.path.exists(os.path.dirname(os.path.abspath(dest))):
                os.mkdir(os.path.dirname(os.path.abspath(dest)))

            os.symlink( os.path.abspath(src), dest )

    def _copy( src, dest ):
        dest = os.path.abspath( os.path.join( PARAMS["web_dir"], dest ) )
        if os.path.exists( dest ): shutil.rmtree( dest )
        if not os.path.exists(src):
            E.warn( "%s does not exist - skipped" % src )
            return
        shutil.copytree( os.path.abspath(src), dest ) 

    # publish export dir via symlinking
    E.info( "linking export directory in %s" % dest_export )
    _link( src_export, dest_export )

    # publish web pages by copying
    E.info( "publishing web pages in %s" % os.path.abspath( os.path.join( PARAMS["web_dir"], dest_report )))
    _copy( os.path.abspath("report/html"), dest_report )

    # substitute links to export and report
    _patterns = [ (re.compile( src_export ), 
                   "http://www.cgat.org/downloads/%(project_id)s/%(dest_export)s" % locals() ), 
                  (re.compile( '(%s)/report' % os.path.join( prefix_project, getProjectName() ) ),
                   "http://www.cgat.org/downloads/%(project_id)s/%(dest_report)s" % locals() ),
                  (re.compile( '(%s)/_static' % curdir),
                   "http://www.cgat.org/downloads/%(project_id)s/%(dest_report)s/_static" % locals() )]
    
    _patterns.extend( patterns )

    for root, dirs, files in os.walk(os.path.join( web_dir, dest_report)):
        for f in files:
            fn = os.path.join( root, f )
            if fn.endswith(".html" ):
                with open( fn ) as inf:
                    data = inf.read()
                for rx, repl in _patterns:
                    data = rx.sub( repl, data )
                outf = open( fn, "w" )
                outf.write( data )
                outf.close()

    if export_files:
        bigwigs, bams, beds = [], [], []

        for targetdir, filenames in export_files.items():
            
            targetdir = os.path.join( web_dir, targetdir )
            if not os.path.exists( targetdir ): os.makedirs( targetdir )

            for src in filenames:
                dest = os.path.join( targetdir, os.path.basename(src) )
                if dest.endswith( ".bam"): bams.append( (targetdir, dest ))
                elif dest.endswith( ".bw"): bigwigs.append( (targetdir, dest ))
                elif dest.endswith( ".bed.gz"): beds.append( (targetdir, dest ))
                dest = os.path.abspath( dest )
                if not os.path.exists( dest ):
                    try:
                        os.symlink( os.path.abspath(src), dest )
                    except OSError, msg:
                        E.warn( "could not create symlink from %s to %s: %s" % \
                                    (os.path.abspath(src), dest, msg))
                        
        # output ucsc links
        with open( "urls.txt", "w" ) as outfile:
            for targetdir, fn in bams: 
                filename = os.path.basename( fn )
                track = filename[:-len(".bam")]
                outfile.write( """track type=bam name="%(track)s" bigDataUrl=http://www.cgat.org/downloads/%(project_id)s/%(targetdir)s/%(filename)s\n""" % locals())

            for targetdir, fn in bigwigs: 
                filename = os.path.basename( fn )
                track = filename[:-len(".bw")]
                outfile.write( """track type=bigWig name="%(track)s" bigDataUrl=http://www.cgat.org/downloads/%(project_id)s/%(targetdir)s/%(filename)s\n""" % locals() )

            for targetdir, fn in beds: 
                filename = os.path.basename( fn )
                track = filename[:-len(".bed.gz")]
                outfile.write( """http://www.cgat.org/downloads/%(project_id)s/%(targetdir)s/%(filename)s\n""" % locals() )
        
        E.info( "UCSC urls are in urls.txt" )

    E.info( "report has been published at http://www.cgat.org/downloads/%(project_id)s/%(dest_report)s" % locals())

def getMakefiles( makefiles, source_directory = "", ignore_missing = False):
    """get all makefiles that are included in a set of makefiles.

    Keep the order of inclusion.
    """

    read_makefiles = set()
    output_makefiles = []

    def __getMakefiles( makefiles ):

        new_makefiles = []
        for makefile in makefiles:
            if makefile in read_makefiles: continue
            print makefile
            if os.path.exists( makefile ):
                fn = makefile
            elif os.path.exists( os.path.join( source_directory, makefile) ):
                fn = os.path.join( source_directory, makefile  )
            else:
                if ignore_missing:
                    continue
                else:
                    raise IOError, "could not find %s in %s" % (makefile, source_directory)
                
            output_makefiles.append( fn )
            infile = open(fn, "r")
            
            for line in infile:
                if re.match("include\s+(\S+)", line):
                    fn = re.search("include\s+(\S+)", line).groups()[0]
                    # add explicitely given files
                    if os.path.exists( fn ):
                        new_makefile = fn
                    else:
                        new_makefile = os.path.basename( fn )
                        # remove any path name variables
                        new_makefile = new_makefile[new_makefile.find(")")+1:]
                    if new_makefile not in read_makefiles:
                        new_makefiles.append( new_makefile )
            infile.close()

            read_makefiles.add( makefile )
            
        if new_makefiles:
            __getMakefiles( new_makefiles )

    __getMakefiles( makefiles )
    
    return output_makefiles

def getScripts( makefiles, source_directory ):
    """extract all python and perl scripts from a set of makefiles."""
    
    scripts = set()
    
    for makefile in makefiles:
        for line in open( makefile,"r"):        
            if re.search( "python", line):
                try:
                    python_scripts = re.search( "python\s+(\S+.py)", line ).groups()
                except AttributeError:
                    continue
                
                for s in python_scripts: scripts.add( ("python", s) )
    
            if re.search( "perl", line):
                try:
                    perl_scripts = re.search( "perl\s+(\S+.pl)", line ).groups()
                except AttributeError:
                    continue
                
                for s in perl_scripts: scripts.add( ("perl", s) )
    
    return scripts

def getModules( modules, scriptdirs, libdirs):
    """extract all imported libraries (modules) from a set of (python) scripts.

    Libraries that are not found are ignored and assumed to be
    installed systemwide.
    """

    read_modules = set()
    system_modules = set()
    
    def __getModules( modules ):
        
        new_modules = set()

        for lib in modules:
            if lib in read_modules or lib in system_modules: continue
            for x in scriptdirs + libdirs:
                if os.path.exists( x + lib ):
                    for line in open( x + lib,"r"):
                        if re.match("import\s+(\S+)", line):
                            ll = re.search("import\s+(\S+)", line).groups()[0]
                            ll = filter( lambda x: x != "", map(lambda x: x.strip(), ll.split(",")))
                            for l in ll:
                                new_modules.add( l + ".py" )
                                
                    read_modules.add( lib )                                
                    break
            else:
                system_modules.add( lib )

        if new_modules:
            __getModules( new_modules )

    __getModules( modules )

    read_modules = read_modules.difference( modules )
    
    return read_modules, system_modules




'''
gtf2tsv.py - compare two genesets
==================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Genomics Genesets GTF Comparison

Purpose
-------

This script compares two genesets in :term:`gtf`-formatted files and output lists
of shared and unique genes.

It outputs the results of the comparison into various sections. The sections are
split into separate output files whose names are determined by the ``--output-filename-pattern`` 
option. The sections are:

genes_ovl
   Table with overlapping genes

genes_total
   Summary statistic of overlapping genes

genes_uniq1
   List of genes unique in set 1

genes_uniq2
   List of genes unique in set 2

Usage
-----

Example::

   python gtfs2tsv.py a.gtf b.gtf

Type::

   python gtfs2tsv.py --help

for command line help.

Command line options
--------------------

'''
import sys
import string
import re
import os
import optparse
import collections
import CGAT.Experiment as E
import CGAT.GTF as GTF
import CGAT.IOTools as IOTools
import bx.intervals.intersection

def GetNextLine( infile ):
    for line in infile:
        if line[0] == "#": continue
        return line
    return None

class Counts:

    mPercentFormat = "%.2f"

    def __init__(self, add_percent ):

        self.nleft, self.nright, self.noverlap = 0, 0, 0
        self.nunique_left = 0
        self.nunique_right = 0
        self.nidentical = 0
        self.nhalf = 0
        self.nsplit_left = 0
        self.nsplit_right = 0
        self.mAddPercent = add_percent
        
    def __add__(self, other):
        self.nleft += other.nleft
        self.nright += other.nright
        self.noverlap += other.noverlap
        self.nunique_left += other.nunique_left
        self.nunique_right += other.nunique_right
        self.nidentical += other.nidentical
        self.nhalf += other.nhalf
        self.nsplit_left += other.nsplit_left
        self.nsplit_right += other.nsplit_right
        return self
    
    def getHeader( self ):
        h = "total_left\ttotal_right\tnoverlap\tnidentical\tnhalf\tunique_left\tunique_right\tsplit_left\tsplit_right"        
        if self.mAddPercent:
            h += "\t" + self.getHeaderPercent()
        return h

    def __str__( self ):
        h = "\t".join(map(str, \
                             (self.nleft, self.nright,
                              self.noverlap, self.nidentical, self.nhalf,
                              self.nunique_left, self.nunique_right,
                              self.nsplit_left, self.nsplit_right)))
        if self.mAddPercent:
            h += "\t" + self.asPercent()
            
        return h 

    def getHeaderPercent( self ):
        return "\t".join( map( lambda x: "pl%s\tpr%s" % (x,x), ("overlap", "identical", "half", "unique", "split")))
    
    def asPercent( self ):
        return "\t".join( map( lambda x: self.mPercentFormat % (100.0 * x),
                               ( float(self.noverlap) / self.nleft,
                                 float(self.noverlap) / self.nright,    
                                 float(self.nidentical) / self.nleft,
                                 float(self.nidentical) / self.nright,    
                                 float(self.nhalf) / self.nleft,
                                 float(self.nhalf) / self.nright,    
                                 float(self.nunique_left) / self.nleft,
                                 float(self.nunique_right) / self.nright,    
                                 float(self.nsplit_left) / self.nleft,
                                 float(self.nsplit_right) / self.nright )))

def getFile( options, section ):

    if options.output_filename_pattern:
        outfile = IOTools.openFile(options.output_filename_pattern % section, "w")
        E.info( "output for section '%s' goes to file %s" % (section, options.output_filename_pattern % section) )
    else:
        outfile = options.stdout
        outfile.write( "## section: %s\n" % section)
    return outfile

def writeDiff( outfile, symbol, genes ):

    for gene in genes:
        for exon in gene:
            outfile.write("%s\t%s\n" % (symbol, str(exon)))

def main( argv = None ):

    if not argv: argv = sys.argv

    parser = E.OptionParser( version = "%prog version: $Id: diff_gtf.py 2781 2009-09-10 11:33:14Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-e", "--write-equivalent", dest="write_equivalent", 
                      help="write equivalent entries [default=%default].", action="store_true"  )

    parser.add_option("-f", "--write-full", dest="write_full",
                      help="write full gff entries [default=%default].", action="store_true"  )

    parser.add_option("-o", "--format=", dest="format",
                      help="output format [flat|multi-line] [default=%default]" )

    parser.add_option("-p", "--add-percent", dest="add_percent", action="store_true",
                      help="add percentage columns [default=%default]." )

    parser.add_option("-s", "--ignore-strand", dest="ignore_strand", action="store_true",
                      help="ignore strand information [default=%default]." )

    parser.set_defaults(
        write_equivalent = False,
        write_full = False,
        format="flat",
        add_percent=False,
        ignore_strand = False,
        as_gtf = False,
        )

    (options, args) = E.Start( parser, argv, add_output_options = True )

    if len(args) != 2:
        raise ValueError( "two arguments required" )

    input_filename1, input_filename2 = args

    ## duplicated features cause a problem. Make sure
    ## features are non-overlapping by running
    ## gff_combine.py on GFF files first.

    E.info( "reading data started" )

    idx, genes2 = {}, set()
    for e in GTF.readFromFile( IOTools.openFile( input_filename2, "r" ) ):
        genes2.add( e.gene_id )
        if e.contig not in idx: idx[e.contig] = bx.intervals.intersection.Intersecter()
        idx[e.contig].add_interval( bx.intervals.Interval(e.start,e.end,value=e) )

    overlaps_genes = []

    E.info( "reading data finished: %i contigs" % len(idx))

    # outfile_diff and outfile_overlap not implemented
    # outfile_diff = getFile( options, "diff" )
    # outfile_overlap = getFile( options, "overlap" )
    overlapping_genes = set()

    genes1 = set()

    # iterate over exons
    with IOTools.openFile( input_filename1, "r" ) as infile:
        for this in GTF.iterator( infile ):

            genes1.add( this.gene_id )

            try:
                intervals = idx[this.contig].find( this.start, this.end )
            except KeyError:
                continue

            others = [ x.value for x in intervals ]
            for other in others:
                overlapping_genes.add( (this.gene_id, other.gene_id) )

            # check for identical/half-identical matches
            output = None
            for other in others:
                if this.start == other.start and this.end == other.end:
                    output, symbol = other, "="
                    break
            else:
                for other in others:
                    if this.start == other.start or this.end == other.end:
                        output, symbol = other, "|"
                        break
                else:
                    symbol = "~"

    # if outfile_diff != options.stdout: outfile_diff.close()
    # if outfile_overlap != options.stdout: outfile_overlap.close()

    outfile = None
    ##################################################################
    ##################################################################
    ##################################################################
    ## print gene based information
    ##################################################################    
    if overlapping_genes:
        outfile = getFile( options, "genes_ovl" )
        outfile.write ("gene_id1\tgene_id2\n" )
        for a, b in overlapping_genes: outfile.write( "%s\t%s\n" % (a,b) )
        if outfile != options.stdout: outfile.close()

        outfile_total = getFile(options, "genes_total" )
        outfile_total.write("set\tngenes\tnoverlapping\tpoverlapping\tnunique\tpunique\n" )

        outfile = getFile( options, "genes_uniq1" )
        b = set( [ x[0] for x in overlapping_genes ] )
        d = genes1.difference(b)
        outfile.write ("gene_id1\n" )
        outfile.write( "\n".join( d ) + "\n" )
        if outfile != options.stdout: outfile.close()
        outfile_total.write( "%s\t%i\t%i\t%5.2f\t%i\t%5.2f\n" % (
                os.path.basename(input_filename1), len(genes1), len(b), 100.0 * len(b) / len(a),
                len(d), 100.0 * len(d) / len(genes1) ) )

        outfile = getFile( options, "genes_uniq2" )
        b = set( [ x[1] for x in overlapping_genes ] )
        d = genes2.difference(b)
        outfile.write ("gene_id2\n" )
        outfile.write( "\n".join( d ) + "\n" )
        if outfile != options.stdout: outfile.close()

        outfile_total.write( "%s\t%i\t%i\t%5.2f\t%i\t%5.2f\n" % (
                os.path.basename(input_filename2), len(genes2), len(b), 100.0 * len(b) / len(a),
                len(d), 100.0 * len(d) / len(genes2) ) )
        if outfile_total != options.stdout: outfile_total.close()
        
    E.Stop()    

if __name__ == "__main__":
    sys.exit( main() )



'''
fastq2table.py - compute stats on reads in fastq files
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Genomics NGS Sequences FASTQ Annotation

Purpose
-------

This script iterates over a fastq file and outputs
summary statistics for each read.

The output is a tab-delimited text file with the following columns:

+----------+----------------------------------------+
|*Column*  |*Content*                               |
+----------+----------------------------------------+
|read      |read identifier present in input fastq  |
|          |file                                    |
+----------+----------------------------------------+
|nfailed   |number of reads that fall below Q10     |
+----------+----------------------------------------+
|nN        |number of ambiguous base calls (N)      |
+----------+----------------------------------------+
|nval      |number of bases in the read             |
+----------+----------------------------------------+
|min       |minimum base quality score for the read |
+----------+----------------------------------------+
|max       |maximum base quality for the read       |
+----------+----------------------------------------+
|mean      |mean base quality for the read          |
+----------+----------------------------------------+
|median    |median base quality for the read        |
+----------+----------------------------------------+
|stddev    |standard devitation of quality scores   |
|          |for the read                            |
+----------+----------------------------------------+
|sum       |sum of quality scores for the read      |
+----------+----------------------------------------+
|q1        |25th percentile of quality scores for   |
|          |the read                                |
+----------+----------------------------------------+
|q3        |25th percentile of quality scores for   |
|          |the read                                |
+----------+----------------------------------------+

Usage
-----

Example::

   python fastq2table.py --guess-format=sanger < in.fastq > out.tsv

In this example we know that our data have quality scores formatted as sanger. Given that
illumina-1.8 quality scores are highly overlapping with sanger, this option defaults to
sanger qualities. In default mode the script may not be able to distinguish
highly overlapping sets of quality scores.

Type::

   python fastq2table.py --help

for command line help.

Command line options
--------------------

'''

import os
import sys
import re
import optparse
import math
import random

import CGAT.Experiment as E
import CGAT.Stats as Stats
import CGAT.Fastq as Fastq

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option( "--guess-format", dest="guess_format", type="choice",
                      choices = ('sanger', 'solexa', 'phred64', 'illumina-1.8', 'integer' ),
                      help="The default behaviour of the script is to guess the quality format of the input fastq file. The user can specify \
                            the quality format of the input file using the --format option. The script will use this format if the \
                            sequence qualities are ambiguous.[default=%default]."  )

    parser.add_option("-f", "--change-format", dest="change_format", type="choice",
                      choices = ('sanger', 'solexa', 'phred64', 'illumina-1.8', 'integer' ),
                      help="The script will guess the quality format of the input file and convert \
                            quality scores to the destination format unless --format is specified [default=%default]."  )

    parser.set_defaults(
        change_format = None,
        guess_format = None,
        min_quality = 10,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    c = E.Counter()
    
    if options.change_format:
        iterator = Fastq.iterate_convert( options.stdin, 
                                          format = options.change_format,
                                          guess = options.guess_format )
    else:
        iterator = Fastq.iterate_guess( options.stdin,
                                        guess = options.guess_format )

    options.stdout.write( "read\tnfailed\tnN\t%s\n" % ("\t".join(Stats.Summary().getHeaders()) ) )

    min_quality = options.min_quality

    for record in iterator:
        c.input += 1
        quals = record.toPhred()
        nfailed = len( [x for x in quals if x < min_quality ] )
        nns = record.seq.count("N") + record.seq.count(".")
        options.stdout.write( "%s\t%i\t%i\t%s\n" % (record.identifier,
                                                    nfailed,
                                                    nns,
                                                    str(Stats.Summary(quals))
                                                    ) )
        c.output += 1


    ## write footer and output benchmark information.
    E.info( "%s" % str(c) )
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



'''
diff_fasta.py - compare contents of two fasta files
===================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Genomics Sequences FASTA Comparison

Purpose
-------

This script takes two sets of fasta sequences and matches the
identifiers. It then compares the sequences with the same
identifiers and outputs

   * which sequences are missing
   * which sequences are identical
   * which sequences are prefixes/suffixes of each other

Depending on the option ``--output`` the following are output:

diff
   identifiers of sequences that are different

seqdiff
   sequences of sequences that are different

missed
   seqences that are missing from set or the other

This script is of specialized interest and has been used
in the past to check if ENSEMBL gene models had been
correctly mapped into a database schema.

Usage
-----

Example::

   python diff_fasta.py a.fasta b.fasta 

Type::

   python diff_fasta.py --help

for command line help.

Command line options
--------------------

'''
import os
import sys
import string
import re
import getopt
import optparse

import CGAT.Genomics as Genomics
import CGAT.Experiment as E
import CGAT.IOTools as IOTools

def MapIdentifiers( seqs, pattern):

    rx = re.compile(pattern)
    
    for k,s in seqs.items():
        try:
            nk = rx.search(k).groups()[0]
        except AttributeError:
            raise ValueError( "identifier can not be parsed from '%s' pattern='%s'" % (k, pattern) )
            
        del seqs[k]
        seqs[nk] = s
        
def main( argv = None ):

    if argv == None: argv = sys.argv 

    parser = E.OptionParser( version = "%prog version: $Id: diff_fasta.py 2781 2009-09-10 11:33:14Z andreas $",
                             usage = globals()["__doc__"] )

    parser.add_option("-s", "--correct-gap-shift", dest="correct_shift", action="store_true",
                      help="correct gap length shifts in alignments. Requires alignlib_lite.py_ "
                      "[%default]")
    parser.add_option("-1", "--pattern1", dest="pattern1", type="string",
                      help="pattern to extract identifier from in identifiers1. "
                      "[%default]")
    parser.add_option("-2", "--pattern2", dest="pattern2", type="string",
                      help="pattern to extract identifier from in identifiers2. "
                      "[%default]")

    parser.add_option("-o", "--output", dest="output", type="choice", action="append",
                      choices=("diff", "missed", "seqdiff"),
                      help="what to output [%default]")
                      
    parser.set_defaults( correct_shift = False,
                         pattern1 = "(\S+)",
                         pattern2 = "(\S+)",
                         output = [] ) 
                         

    (options, args) = E.Start( parser )

    if len(args) != 2:
        raise ValueError( "two files needed to compare." )

    if options.correct_shift:
        try:
            import alignlib_lite
        except ImportError:
            raise ImportError("option --correct-shift requires alignlib_lite.py_ but alignlib not found" )

    seqs1 = Genomics.ReadPeptideSequences( IOTools.openFile(args[0], "r") )
    seqs2 = Genomics.ReadPeptideSequences( IOTools.openFile(args[1], "r") )    

    if not seqs1: raise ValueError( "first file %s is empty." % (args[0] ) )
    if not seqs2: raise ValueError( "second file %s is empty." % (args[1] ) )
    
    MapIdentifiers( seqs1, options.pattern1 )
    MapIdentifiers( seqs2, options.pattern2 )    
    
    nsame = 0
    nmissed1 = 0
    nmissed2 = 0
    ndiff = 0
    ndiff_first = 0
    ndiff_last = 0
    ndiff_prefix = 0
    ndiff_selenocysteine = 0
    ndiff_masked = 0
    nfixed = 0
    found2 = {}

    write_missed1 = "missed" in options.output
    write_missed2 = "missed" in options.output
    write_seqdiff = "seqdiff" in options.output
    write_diff = "diff" in options.output or write_seqdiff
    
    for k in seqs1:
        if k not in seqs2:
            nmissed1+=1
            if write_missed1:
                options.stdout.write("---- %s ---- %s\n" % (k, "missed1") )
            continue
        
        found2[k] = 1
        
        s1 = seqs1[k].upper()
        s2 = seqs2[k].upper()
        m = min(len(s1), len(s2))
        
        if s1 == s2:
            nsame += 1
        else:
            status = "other"
            
            ndiff += 1
            
            if s1[1:] == s2[1:]:
                ndiff_first += 1
                status = "first"
            elif s1[:m] == s2[:m]:
                ndiff_prefix += 1
                status = "prefix"
            elif s1[:-1] == s2[:-1]:
                ndiff_last += 1
                status = "last"
            else:
                if len(s1) == len(s2):
                    # get all differences:
                    # the first and last residues can be different for peptide sequences when comparing
                    # my translations with ensembl peptides.
                    differences = []
                    for x in range(1,len(s1)-1):
                        if s1[x] != s2[x]:
                            differences.append( (s1[x], s2[x]) )

                    l = len(differences)
                    # check for Selenocysteins
                    if len( filter( lambda x: x[0] == "U" or x[1] == "U", differences)) == l:
                        ndiff_selenocysteine += 1
                        status = "selenocysteine"
                        
                    # check for masked residues
                    elif len( filter( lambda x: x[0] in "NX" or x[1] in "NX", differences)) == l:
                        ndiff_masked += 1
                        status = "masked"
                        
            ## correct for different gap lengths
            if options.correct_shift:
                        
                map_a2b = alignlib_lite.py_makeAlignmentVector()

                a, b = 0, 0
                keep = False
                
                x = 0
                while x < m and not (a == len(s1) and b == len(s2)):
                    try:
                        if s1[a] != s2[b]:
                            while s1[a] == "N" and s2[b] != "N":
                                a += 1
                            while s1[a] != "N" and s2[b] == "N":
                                b += 1
                            
                            if s1[a] != s2[b]:
                                break
                    except IndexError:
                        print "# index error for %s: x=%i, a=%i, b=%i, l1=%i, l2=%i" % (k, x, a, b, len(s1), len(s2))
                        break

                    a += 1
                    b += 1
                    map_a2b.addPairExplicit( a, b, 0.0 )
                    ## check if we have reached the end:
                else:
                    keep = True
                    nfixed += 1
                    f = alignlib_lite.py_AlignmentFormatEmissions( map_a2b)
                    print "fix\t%s\t%s" % (k, str(f))

                if not keep:
                    print "# warning: not fixable: %s" % k
                    
            if write_diff:
                options.stdout.write("---- %s ---- %s\n" % (k, status) )

            if write_seqdiff:
                options.stdout.write( "< %s\n> %s\n" % (seqs1[k], seqs2[k]) )

    for k in seqs2.keys():
        if k not in found2:
            nmissed2 += 1
            if write_missed2:
                options.stdout.write("---- %s ---- %s\n" % (k, "missed2") )

    options.stdlog.write( """# Legend:
# seqs1:          number of sequences in set 1
# seqs2:          number of sequences in set 2
# same:           number of identical sequences
# diff:           number of sequences with differences
# nmissed1:       sequences in set 1 that are not found in set 2
# nmissed2:       sequences in set 2 that are not found in set 1
# Type of sequence differences
# first:          only the first residue is different
# last:           only the last residue is different
# prefix:         one sequence is prefix of the other
# selenocysteine: difference due to selenocysteines
# masked:         difference due to masked residues
# fixed:          fixed differences
# other:          other differences
""")

    E.info( "seqs1=%i, seqs2=%i, same=%i, ndiff=%i, nmissed1=%i, nmissed2=%i" %\
                (len(seqs1), len(seqs2), nsame, ndiff, nmissed1, nmissed2) )
    E.info( "ndiff=%i: first=%i, last=%i, prefix=%i, selenocysteine=%i, masked=%i, fixed=%i, other=%i" %\
                (ndiff, ndiff_first, ndiff_last, ndiff_prefix, ndiff_selenocysteine, ndiff_masked, nfixed,
                 ndiff - ndiff_first - ndiff_last - ndiff_prefix - ndiff_selenocysteine - ndiff_masked - nfixed) )
    
    E.Stop()
        
if __name__ == "__main__":
    sys.exit(main( sys.argv ))



'''
bam2bam.py - modify bam files
=============================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Genomics NGS BAM Manipulation

Purpose
-------

This script reads a :term:`bam` formatted file from stdin and outputs a
modified :term:`bam` formatted file on stdout.

.. note::
   You need to redirect logging information to a file or turn it off
   via -v 0 in order to get a valid sam/bam file.

Usage
-----

For example::

   python bam2bam.py --filter=mapped < in.bam > out.bam

will remove all unmapped reads from the bam-file. To remove
unmapped reads from multiple bam-files, try::

   python bam2bam.py --inplace --filter=mapped *.bam

Type::

   python bam2bam.py --help

for command line help.

Documentation
-------------

The script implements the following methods:

``set-nh``
   set the NH flag. Some tools (bowtie, bwa) do not set the NH flag. 
   If set, this option will set the NH flag (for mapped reads).
   This option requires the bam/sam file to be sorted by read name.

``unset-unmapped_mapq`` 
   some tools set the mapping quality of unmapped reads. This
   causes a violation in the Picard tools.

``remove-better``
   remove alignments that are worse than alignments present in
   an alternative :term:`bam` formatted file (``--filter-bam``). 

``filter``
   remove alignments based on a variety of flags.

``strip`` 
   remove the sequence and/or quality scores from a bam-file

``set-sequence``
   set the sequence and quality scores in the bam file to some 
   dummy sequence. Necessary for some tools that can not work
   with bam-files without sequence.

By default, the script works from stdin and outputs to stdout.
If the ``--inplace option`` is given, the script will modify
bam-files given as command line arguments in-place - a temporary
copy will be written to :file:`/tmp` and once completed copied
over the original file. The script will automatically re-index
the modified files.

Command line options
--------------------

'''

import os
import sys
import re
import optparse
import collections
import itertools
import tempfile
import shutil

import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import pysam

try:
    import pyximport
    pyximport.install(build_in_temp=False)
    import _bam2bam
except ImportError:
    import CGAT._bam2bam as _bam2bam

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                             usage = globals()["__doc__"] )

    parser.add_option( "--set-nh", dest="set_nh", action="store_true",
                       help = "sets the NH flag. The file needs to be sorted by readname [%default]" )

    parser.add_option( "--unset-unmapped-mapq", dest="unset_unmapped_mapq", action="store_true",
                       help = "sets the mapping quality of unmapped reads to 0 [%default]" )

    parser.add_option( "--set-sequence", dest="set_sequence", action="store_true",
                       help = "sets the sequence to 'A's (a valid base) and the quality to 'F's"
                       ",which is defined in all fastq scoring schemes "
                       "[%default]" )

    parser.add_option( "--strip", dest="strip", type="choice",
                       choices = ("sequence", "quality" ),
                       help = "remove parts of the bam-file. Note that stripping the sequence will "
                       "also strip the quality values [%default]" )

    parser.add_option( "--filter", dest="filter", action="append", type="choice",
                       choices=('NM', 'CM', 'mapped', 'unique', "non-unique" ),
                       help = "filter bam file. The option denotes the property that is  "
                       "used to determine better match [%default]" )

    parser.add_option( "--reference-bam", dest="reference_bam", type="string",
                       help = "bam-file to filter with [%default]" )
    
    parser.add_option( "--sam", dest="output_sam", action="store_true",
                       help = "output in sam format [%default]" )

    parser.add_option( "--inplace", dest="inplace", action="store_true",
                       help = "modify bam files in-place. Bam files need to be given "
                       "as arguments. Temporary bam files are written to /tmp [%default]" )

    parser.set_defaults(
        filter = [],
        set_nh = False,
        unset_unmapped_mapq = False,
        output_sam = False,
        reference_bam = None,
        strip = None,
        set_sequence = False,
        inplace = False,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    if options.inplace:
        bamfiles = args
        if len(bamfiles) == 0:
            raise ValueError("please one or more bam-files as command line arguments" )

        if "-" in bamfiles:
            raise ValueError("can not read from stdin if ``--inplace`` is selected" )
    else:
        bamfiles = ["-"]

    for bamfile in bamfiles:

        # reading bam from stdin does not work with only the "r" tag
        pysam_in = pysam.Samfile( bamfile, "rb" )

        if bamfile == "-":
            if options.output_sam:
                pysam_out = pysam.Samfile( "-", "wh", template = pysam_in )
            else:
                pysam_out = pysam.Samfile( "-", "wb", template = pysam_in )
        else:
            tmpfile = tempfile.NamedTemporaryFile( delete=False, prefix = "ctmp" )
            tmpfile.close()

            E.debug("writting temporary bam-file to %s" % tmpfile.name )
            pysam_out = pysam.Samfile( tmpfile.name, "wb", template = pysam_in )

        if options.filter:

            remove_mismatches, colour_mismatches = False, False

            if "NM" in options.filter:
                remove_mismatches = True

            elif "CM" in options.filter:
                remove_mismatches = True
                colour_mismatches = True

            if remove_mismatches:
                if not options.reference_bam:
                    raise ValueError( "requiring reference bam file for removing by mismatches")

                pysam_ref = pysam.Samfile( options.reference_bam, "rb" )
            else:
                pysam_ref = None

            # filter and flags are the opposite way around
            c = _bam2bam.filter_bam( pysam_in, pysam_out, pysam_ref,
                                     remove_nonunique = "unique" in options.filter,
                                     remove_unique = "non-unique" in options.filter,
                                     remove_contigs = None,
                                     remove_unmapped = "mapped" in options.filter,
                                     remove_mismatches = remove_mismatches,
                                     colour_mismatches = colour_mismatches )

            options.stdlog.write( "category\tcounts\n%s\n" % c.asTable() )

        else:
            # set up the modifying iterators
            it = pysam_in.fetch( until_eof = True )

            if options.unset_unmapped_mapq:
                def unset_unmapped_mapq( i ):
                    for read in i:
                        if read.is_unmapped:
                            read.mapq = 0
                        yield read
                it = unset_unmapped_mapq( it )

            if options.set_nh and False:
                def set_nh( i ):

                    for key, reads in itertools.groupby( i, lambda x: x.qname ):
                        l = list(reads)
                        nh = len(l)
                        for read in l:
                            if not read.is_unmapped:
                                t = dict( read.tags )
                                t['NH'] = nh
                                read.tags = list(t.iteritems())
                            yield read
                it = set_nh( it )

            if options.set_sequence:
                def set_sequence( i ):
                    for read in i:
                        # can't get at length of unmapped reads
                        if read.is_unmapped: 
                            read.seq = "A"
                            read.qual = "F"
                        else:
                            read.seq = "A" * read.inferred_length
                            read.qual = "F" * read.inferred_length

                        yield read
                it = set_sequence( it )

            if options.strip != None:
                def strip_sequence( i ):
                    for read in i:
                        read.seq = None
                        yield read

                def strip_quality( read ):
                    for read in i:
                        read.qual = None
                        yield read

                if options.strip == "sequence":
                    it = strip_sequence( it )
                elif options.strip == "quality":
                    it = strip_quality( it )

            if options.set_nh:
                it = _bam2bam.SetNH( it )

            # read and output
            for read in it: 
                pysam_out.write( read )

            pysam_in.close()
            pysam_out.close()

        if options.inplace:
            # move new file over original copy
            shutil.move( tmpfile.name, bamfile )
            # re-index
            pysam.index( bamfile )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




'''
fastqs2fastqs.py - reconcile reads from a pair of fastq files
=============================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Genomics NGS FASTQ FASTQ Manipulation

Purpose
-------

This script takes two fastq files and outputs two fastq files. The aim
of the script is to reconcile reads from a pair of fastq files. For example,
if reads from each file are independently filtered based on quality scores,
the read representation will vary across files. This script will output reads
that are common to both files.

The two files must be sorted by read identifier.

Usage
-----

Example::

   python fastqs2fastqs.py myReads.1.fastq.gz myReads.2.fastq.gz --reconcile --output-pattern myReads_reconciled.%i.fastq

In this example we take a pair of fastq files, reconcile by read identifier and output 2 new
fastq files named myReads_reconciled.1.fastq.gz myReads_reconciled.2.fastq.gz

Type::

   python fastqs2fastqs.py --help

for command line help.

Command line options
--------------------

'''

import os
import sys
import re
import optparse
import math
import random

import CGAT.IOTools as IOTools
import CGAT.Experiment as E
import CGAT.Fastq as Fastq

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-m", "--method", dest="method", type="choice",
                      choices = ('reconcile', ),
                      help="method to apply [default=%default]."  )
                      
    parser.add_option("-c", "--chop", dest="chop", action="store_true",
                      help="whether or not to trim last character of sequence name. For example sometimes ids in the first \
                            file in the pair will end with \1 and teh second with \2. If --chop is not specified \
                            then the results will be wrong [default=%default]."  )

    parser.add_option("-o", "--output-pattern", dest="output_pattern", type="string",
                      help="pattern for output files [default=%default]."  )

    parser.set_defaults(
        method = "reconcile",
        chop = False,
        output_pattern = "%i.fastq.gz",
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    if len(args) != 2:
        raise ValueError("please supply at least two fastq files on the commandline")

    fn1, fn2 = args
    c = E.Counter()
    
    if options.method == "reconcile":

        def getIds( infile ):
            '''return ids in infile.'''
            aread = infile.readline
            while True:
                l = [aread().rstrip("\r\n") for i in range(4)]
                if not l[0]: break
                r = l[0].split()[0]
                # decide if to chop read number off
                if options.chop: yield r[:-1]
                else: yield r

        def write( outfile, infile, take ):
            '''filter fastq files with ids in take.'''
            aread = infile.readline
            while True:
                l = [aread().rstrip("\r\n") for i in range(4)]
                if not l[0]: break
                r = l[0].split()[0]
                if options.chop: r = r[:-1]
                if r not in take: continue
                outfile.write("\n".join(l) + "\n" )

        E.info( "reading first in pair" )
        inf1 = IOTools.openFile( fn1 )
        ids1 = set( getIds( inf1 ) )

        E.info( "reading second in pair" )
        inf2 = IOTools.openFile( fn2 )
        ids2 = set( getIds( inf2 ) )

        take = ids1.intersection( ids2 )
        
        E.info( "first pair: %i reads, second pair: %i reads, shared: %i reads" % \
                    ( len(ids1),
                      len(ids2),
                      len(take)) )
 
        with IOTools.openFile( options.output_pattern % 1, "w" ) as outf:
            inf = IOTools.openFile( fn1 )
            E.info( "writing first in pair" )
            write( outf, inf, take )

        with IOTools.openFile( options.output_pattern % 2, "w" ) as outf:
            inf = IOTools.openFile( fn2 )
            E.info( "writing second in pair" )
            write( outf, inf, take )
        
    ## write footer and output benchmark information.
    E.info( "%s" % str(c) )
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



'''
gff2histogram.py - compute histograms from intervals in gff or bed format
=========================================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Genomics Intervals GFF Summary

Purpose
-------

This script computes distributions of interval sizes, intersegmental distances
and interval overlap from a list of intervals in :term:`gff` or :term:`bed` 
format. 

The output will be written into separate files. Filenames are given by 
``--ouput-filename-pattern``.

Available methods are:

hist
    Output a histogram of interval sizes and distances between intervals
    in nucleotides.

stats
    Output summary statistics of interval sizes and distances between
    intervals

values
    Output distances, sizes, and overlap values to separate files.

all
    all of the above.

Usage
-----

Example::

   python gff2histogram.py < in.gff

Type::

   python gff2histogram.py --help

for command line help.

Command line options
--------------------

'''
import sys
import string
import re
import optparse

import CGAT.Experiment as E
import CGAT.GTF as GTF
import CGAT.Bed as Bed
import CGAT.Histogram as Histogram
import CGAT.Stats as Stats

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if argv == None: argv = sys.argv

    parser = E.OptionParser( version = "%prog version: $Id: gff2histogram.py 2781 2009-09-10 11:33:14Z andreas $")

    parser.add_option("-b", "--bin-size", dest="bin_size", type="string",
                      help="bin size."  )
    parser.add_option("--min-value", dest="min_value", type="float",
                      help="minimum value for histogram.")
    parser.add_option("--max-value", dest="max_value", type="float",
                      help="maximum value for histogram.")
    parser.add_option("--no-empty-bins", dest="no_empty_bins", action="store_true",
                      help="do not display empty bins.")
    parser.add_option("--with-empty-bins", dest="no_empty_bins", action="store_false",
                      help="display empty bins.")
    parser.add_option("--ignore-out-of-range", dest="ignore_out_of_range", action="store_true",
                      help="ignore values that are out of range (as opposed to truncating them to range border.")
    parser.add_option("--missing", dest="missing_value", type="string",
                      help="entry for missing values [%default]." )
    parser.add_option("--dynamic-bins", dest="dynamic_bins", action="store_true",
                      help="each value constitutes its own bin." )
    parser.add_option( "--format", dest="format", type="choice", 
                       choices=( "gff", "gtf", "bed" ),
                       help="input file format [%default].")
    parser.add_option( "--method", dest="methods", type="choice", action="append",
                       choices=( "all", "hist", "stats", "overlaps", "values" ),
                       help="methods to apply [%default].")
    parser.add_option( "--data", dest="data", type="choice",
                       choices=( "all", "size", "distance" ),
                       help="data to compute [%default].")

    parser.set_defaults(
        no_empty_bins = True,
        bin_size = None,
        dynamic_bins = False,
        ignore_out_of_range = False,
        min_value = None,
        max_value = None,
        nonull = None,
        missing_value = "na",
        output_filename_pattern="%s",
        methods = [],
        data = "all",
        format = "gff",
        )

    (options, args) = E.Start( parser, add_output_options = True )

    if "all" in options.methods:
        options.methods = ("hist", "stats", "overlaps")
        if not options.output_filename_pattern: options.output_filename_pattern = "%s"

    if len(options.methods) == 0:
        raise ValueError( "please provide counting method using --method option" )

    if options.format in ( "gff", "gtf" ):
        gffs = GTF.iterator( options.stdin )
    elif options.format == "bed":
        gffs = Bed.iterator( options.stdin )

    values_between = []
    values_within = []
    values_overlaps = []

    if "overlaps" in options.methods:
        if not options.output_filename_pattern: 
            options.output_filename_pattern = "%s"
        outfile_overlaps = E.openOutputFile( "overlaps" )
    else:
        outfile_overlaps = None

    last = None
    ninput, noverlaps = 0,0
    for this in gffs:
        ninput += 1
        values_within.append( this.end - this.start )

        if last and last.contig == this.contig:
            if this.start < last.end:
                noverlaps += 1
                if outfile_overlaps:
                    outfile_overlaps.write( "%s\t%s\n" % (str(last), str(this)) )
                values_overlaps.append( min(this.end, last.end) - max(last.start, this.start) )
                if this.end > last.end:
                    last = this
                continue
            else:
                values_between.append( this.start - last.end )
                # if this.start - last.end < 10: 
                #     print str(last)
                #     print str(this)
                #     print "=="
                values_overlaps.append( 0 )

        last = this

    if "hist" in options.methods:
        outfile = E.openOutputFile( "hist" )
        h_within = Histogram.Calculate( values_within,
                                        no_empty_bins = options.no_empty_bins,
                                        increment = options.bin_size,
                                        min_value = options.min_value,
                                        max_value = options.max_value,
                                        dynamic_bins = options.dynamic_bins,
                                        ignore_out_of_range = options.ignore_out_of_range )

        h_between = Histogram.Calculate( values_between,
                                         no_empty_bins = options.no_empty_bins,
                                         increment = options.bin_size,
                                         min_value = options.min_value,
                                         max_value = options.max_value,
                                         dynamic_bins = options.dynamic_bins,
                                         ignore_out_of_range = options.ignore_out_of_range )

        if "all" == options.data:
            outfile.write( "residues\tsize\tdistance\n" )
            combined_histogram = Histogram.Combine( [h_within, h_between], missing_value = options.missing_value )
            Histogram.Write( outfile, combined_histogram, nonull = options.nonull )        
        elif options.data == "size":
            outfile.write( "residues\tsize\n" )
            Histogram.Write( outfile, h_within, nonull = options.nonull )        
        elif options.data == "distance":
            outfile.write( "residues\tdistance\n" )
            Histogram.Write( outfile, h_between, nonull = options.nonull )        

        outfile.close()

    if "stats" in options.methods:
        outfile = E.openOutputFile( "stats" )
        outfile.write( "data\t%s\n" % Stats.Summary().getHeader() )
        if options.data in ("size", "all"):
            outfile.write( "size\t%s\n" % str(Stats.Summary(values_within)) )
        if options.data in ("distance", "all"):
            outfile.write( "distance\t%s\n" % str(Stats.Summary(values_between)) )
        outfile.close()

    if "values" in options.methods:
        outfile = E.openOutputFile( "distances" )
        outfile.write( "distance\n%s\n" % "\n".join( map(str, values_between) ) )
        outfile.close()
        outfile = E.openOutputFile( "sizes" )
        outfile.write( "size\n%s\n" % "\n".join( map(str, values_within) ) )
        outfile.close()
        outfile = E.openOutputFile( "overlaps" )
        outfile.write( "overlap\n%s\n" % "\n".join( map(str, values_overlaps) ) )
        outfile.close()

    E.info( "ninput=%i, ndistance=%i, nsize=%i, noverlap=%i" % (ninput, 
                                                                len(values_between),
                                                                len(values_within),
                                                                noverlaps) )

    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



import glob
import sys
import os
import subprocess

########################################################################
#######################################################################
## Check for dependencies
##
## Is there a way to do this more elegantly?
##     1. Run "pip install numpy"
##     2. Wrap inside functions (works for numpy/pysam, but not cython)
try:
    import numpy
except ImportError:
    raise ImportError("the CGAT code collection requires numpy to be installed before running setup.py (pip install numpy)" )

try:
    import Cython
except ImportError:
    raise ImportError("the CGAT code collection requires cython to be installed before running setup.py (pip install cython)" )

try: 
    import pysam
except ImportError:
    raise ImportError("the CGAT code collection requires pysam to be installed before running setup.py (pip install pysam)" )

########################################################################
########################################################################
## Import setuptools
## Use existing setuptools, otherwise try ez_setup.
try:
    import setuptools
except ImportError:
    ## try to get via ez_setup
    ## ez_setup did not work on all machines tested as
    ## it uses curl with https protocol, which is not
    ## enabled in ScientificLinux
    import ez_setup
    ez_setup.use_setuptools()

from setuptools import setup, find_packages, Extension
major, minor = map(int, setuptools.__version__.split(".")[:2])
if major < 1 or (major == 1 and minor < 1):
    raise ImportError("the CGAT code collection requires setuptools 1.1 higher")

from Cython.Distutils import build_ext

########################################################################
########################################################################
IS_OSX = sys.platform == 'darwin'

########################################################################
########################################################################
# collect CGAT version
sys.path.insert( 0, "scripts")
import version

version = version.__version__

###############################################################
###############################################################
# Check for external dependencies
#
# Not exhaustive, simply execute a representative tool from a toolkit.
external_dependencies = ( 
    ("wigToBigWig", "UCSC tools", 255),
    ("bedtools", "bedtools", 0 ),
    )

for tool, toolkit, expected in external_dependencies:
    try:
        from subprocess import DEVNULL # py3k
    except ImportError:
        import os
        DEVNULL = open(os.devnull, 'wb')

    try:
        retcode = subprocess.call( tool, shell = True, stdout=DEVNULL, stderr=DEVNULL )
    except OSError, msg:
        print( "WARNING: depency check for %s failed: %s" % (toolkit, msg ) )

    # UCSC tools return 255 when called without arguments
    if retcode != expected:
        print ( "WARNING: depency check for %s(%s) failed, error %i" % \
                    (toolkit, tool, retcode ) )

###############################################################
###############################################################
# Define dependencies 
#
# Perform a CGAT Code Collection Installation
INSTALL_CGAT_CODE_COLLECTION = True

major, minor1, minor2, s, tmp = sys.version_info

if major==3:
    raise SystemExit("""CGAT is not fully python3 compatible""")

if (major==2 and minor1<7) or major<2:
    raise SystemExit("""CGAT requires Python 2.7 or later.""")


dependencies = open( "requires.txt" ).readlines() 
dependencies = [x[:-1] for x in dependencies if x.startswith("#")]

if major==2:
    dependencies.extend( [ 'web.py>=0.37',
                           'xlwt>=0.7.4', 
                           'matplotlib-venn>=0.5' ] )
elif major==3:
    pass

if INSTALL_CGAT_CODE_COLLECTION:
    cgat_packages= find_packages( exclude=["CGATPipelines*", "scripts*"])
else:
    cgat_packages= find_packages( exclude=["scripts*"])

# rename scripts to CGATScripts
cgat_packages.append( "CGATScripts" )

cgat_package_dirs = { 'CGAT': 'CGAT',
                      'CGATScripts' : 'scripts',
                      'CGATPipelines': 'CGATPipelines' }

##########################################################
##########################################################
# Classifiers
classifiers="""
Development Status :: 3 - Alpha
Intended Audience :: Science/Research
Intended Audience :: Developers
License :: OSI Approved
Programming Language :: Python
Topic :: Software Development
Topic :: Scientific/Engineering
Operating System :: POSIX
Operating System :: Unix
Operating System :: MacOS
"""

##########################################################
##########################################################
## Extensions
# Connected components cython extension
Components = Extension(
     'CGAT.Components',                   
     [ 'CGAT/Components/Components.pyx',        
       'CGAT/Components/connected_components.cpp',],
     library_dirs=[],
     libraries=[],              
     language="c++",               
     )

# Nested containment lists
NCL = Extension(
    "CGAT.NCL.cnestedlist",                   
    [ "CGAT/NCL/cnestedlist.pyx",    
      "CGAT/NCL/intervaldb.c" ],
      library_dirs=[],
      libraries=[],
      language="c",
    )

# Nubiscan motif mapping
Nubiscan = Extension(
    "CGAT.Nubiscan.cnubiscan",                   
    [ 'CGAT/Nubiscan/cnubiscan.pyx'],
    library_dirs=[],
    libraries=[],            
    include_dirs = [numpy.get_include()], 
    language="c",               
 )

# Automatically build script extensions
pyx_files = glob.glob( "scripts/*.pyx" )
script_extensions = []
pysam_dirname = os.path.dirname( pysam.__file__ )
include_dirs = [ numpy.get_include() ] + pysam.get_include()

if IS_OSX:
    # linking against bundles does no work (and apparently is not needed)
    # within OS X
    extra_link_args = []
else:
    extra_link_args = [ os.path.join( pysam_dirname, "csamtools.so")]

for pyx_file in pyx_files:
    script_name = os.path.basename( pyx_file )
    script_prefix = script_name[:-4]
    script_extensions.append( 
        Extension( "CGAT.%s" % (script_prefix),
                   sources = [pyx_file],
                   extra_link_args = extra_link_args,
                   include_dirs = include_dirs,
                   define_macros = pysam.get_defines() )
        )

setup(## package information
    name = 'CGAT',
    version= version,
    description = 'CGAT : the Computational Genomics Analysis Toolkit',
    author = 'Andreas Heger',
    author_email = 'andreas.heger@gmail.com',
    license = "BSD",
    platforms = ["any",],
    keywords="computational genomics",
    long_description='CGAT : the Computational Genomics Analysis Toolkit',
    classifiers = filter(None, classifiers.split("\n")),
    url="http://www.cgat.org/cgat/Tools/",
    ## package contents
    packages = cgat_packages, 

    package_dir= cgat_package_dirs,
    # package_data = { 'CGATScripts' : ['./scripts/*.py', './scripts/*.pyx', 
    #                                   './scripts/*.pyxbld', './scripts/*.pl' ],
    #                },

    include_package_data = True,

    entry_points = {
        'console_scripts': ['cgat = CGATScripts.cgat:main' ]
        },

    ## dependencies
    install_requires=dependencies,
    ## extension modules
    ext_modules=[Components, NCL, Nubiscan] + script_extensions,
    cmdclass = {'build_ext': build_ext},
    ## other options
    zip_safe = False,
    test_suite = "tests",
    )




"""
diff_chains.py - compare to chain formatted files
=================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Genomics GenomeAlignment CHAIN Comparison

Purpose
-------

compare two `chain <http://www.breyer.com/ucsc/htdocs/goldenPath/help/chain.html>`_ 
formatted files.

Outputs a table with the following columns:

+-----------+-------------------------------+
|*Column*   |*Content*                      |
+-----------+-------------------------------+
|contig1    |contig name                    |
+-----------+-------------------------------+
|contig2    |contig name                    |
+-----------+-------------------------------+
|strand     |strand                         |
+-----------+-------------------------------+
|mapped1    |mapped residues                |
+-----------+-------------------------------+
|identical1 |identically mapped residues    |
+-----------+-------------------------------+
|different1 |differently mapped residues    |
+-----------+-------------------------------+
|unique1    |residues mapped only from set1 |
+-----------+-------------------------------+
|pmapped1   |percentage of mapped residues  |
+-----------+-------------------------------+
|pidentical1|percentage of identically      |
|           |mapped residues                |
+-----------+-------------------------------+
|pdifferent1|percentage of differently      |
|           |mapped residues                |
+-----------+-------------------------------+

Similar columns exist for data set 2

Usage
-----

Type::

   python diff_chains.py hg19ToMm10v1.chain.over.gz hg19ToMm10v2.chain.over.gz

This will compare the locations that regions within the genome hg19 map to between
two different mappings to the genome mm10.

Type::

   python diff_chains.py --help

for command line help.

Command line options
--------------------

""" 

import os
import sys
import re
import optparse
import collections

import CGAT.IOTools as IOTools
import CGAT.Experiment as E
import alignlib_lite

def chain_iterator( infile ):
    lines = []
    for line in infile:

        if line.startswith("#"): continue
        if line.strip() == "": continue
        if line.startswith("chain"):
            if lines: yield lines
            lines = []
        lines.append( line )

    yield lines

def validateChain( infile ):
    '''validate a chain file.

    No overlapping target coordinates.
    '''
    
    pairs_t2q = collections.defaultdict( alignlib_lite.py_makeAlignmentBlocks )
    pairs_q2t = collections.defaultdict( alignlib_lite.py_makeAlignmentBlocks )

    
    for lines in chain_iterator( infile ):
        
        ( _, 
          _, 
          target_contig,
          target_length,
          target_strand,
          target_start,
          target_end,
          query_contig,
          query_length,
          query_strand,
          query_start, 
          query_end,
          alignment_id ) = lines[0][:-1].split()

        (query_start, 
         query_end,
         query_length,
         target_start, 
         target_end,
         target_length) = \
         [ int(x) for x in 
           (query_start, 
            query_end,
            query_length,
            target_start, 
            target_end,
            target_length) ]
         
        # target_strand is always positive
        assert( target_strand == "+" )
        
        map_target2query = pairs_t2q[ target_contig ]
        map_query2target = pairs_q2t[ query_contig ]

        qstart, tstart = query_start, target_start
        
        for line in lines[1:-1]:
            size, dt, dq = [int(x) for x in line[:-1].split() ]
            map_target2query.addDiagonal( tstart,
                                          tstart + size,
                                          0 )
            map_query2target.addDiagonal( qstart,
                                          qstart + size, 
                                          0 )

            qstart += size + dq
            tstart += size + dt

        size = int(lines[-1][:-1])
        map_target2query.addDiagonal( tstart,
                                      tstart + size,
                                      0 )
        map_query2target.addDiagonal( qstart,
                                      qstart + size,
                                      0 )

    try: 
        x = map_query2target.mapRowToCol(0)
    except RuntimeError:
        E.info( "query is not unique - this is ok." )

    try:
        x = map_target2query.mapRowToCol(0)
    except RuntimeError:
        E.info( "target is not unique" )
        return False
    
    return True

def buildPairs( infile ):
    '''read a chain file.

    build target2query alignments.
    The target is always on the positive strand.
    '''
    pairs = collections.defaultdict( alignlib_lite.py_makeAlignmentBlocks )

    def chain_iterator( infile ):
        lines = []
        for line in infile:
            
            if line.startswith("#"): continue
            if line.strip() == "": continue
            if line.startswith("chain"):
                if lines: yield lines
                lines = []
            lines.append( line )
            
        yield lines

    for lines in chain_iterator( infile ):
        
        ( _, 
          _, 
          target_contig,
          target_length,
          target_strand,
          target_start,
          target_end,
          query_contig,
          query_length,
          query_strand,
          query_start, 
          query_end,
          alignment_id ) = lines[0][:-1].split()

        (query_start, 
         query_end,
         query_length,
         target_start, 
         target_end,
         target_length) = \
         [ int(x) for x in 
           (query_start, 
            query_end,
            query_length,
            target_start, 
            target_end,
            target_length) ]
         
        # target_strand is always positive
        assert( target_strand == "+" )
        
        map_target2query = pairs[ (target_contig, query_contig, query_strand ) ]

        qstart, tstart = query_start, target_start
        
        for line in lines[1:-1]:
            size, dt, dq = [int(x) for x in line[:-1].split() ]
            map_target2query.addDiagonal( tstart,
                                          tstart + size,
                                          qstart - tstart )
            qstart += size + dq
            tstart += size + dt

        size = int(lines[-1][:-1])

        map_target2query.addDiagonal( tstart,
                                      tstart + size,
                                      qstart - tstart )


    return pairs

DiffResult = collections.namedtuple( "DiffResult","total same different unique")
def compareChains( pairs1, pairs2 ):
    '''compare chains in pairs1 versus those in pairs2'''
    
    result = {}
    for key1, chain1 in pairs1.iteritems():
        E.debug( "comparing %s" % str(key1) )

        ntotal = chain1.getNumAligned()

        if key1 not in pairs2:
            result[ key1 ] = DiffResult._make( (ntotal, 0, 0, ntotal) )
            continue

        chain2 = pairs2[key1]
        nsame = alignlib_lite.py_getAlignmentIdentity( chain1, chain2, alignlib_lite.py_RR )
        noverlap = alignlib_lite.py_getAlignmentOverlap( chain1, chain2, alignlib_lite.py_RR )
        ndifferent = noverlap - nsame
        nunique = ntotal - noverlap

        result[ key1 ] = DiffResult._make( (ntotal, nsame, ndifferent, nunique ) )

    return result

def outputMismatches( pairs1, pairs2, 
                      output_mismatches = False, 
                      output_unique = False,
                      output_matches = False ):
    '''output mismatches.

    This is a very slow operation.
    '''

    outfile = sys.stdout
    

    for key1, chain1 in pairs1.iteritems():
        E.debug( "comparing %s" % str(key1) )

        if key1 not in pairs2:
            outfile.write( "%s\t%s\t%s\t%i\t%i\t-1\n" %
                           ( key1 + (chain1.getRowFrom(), chain2.getRowTo()) ) )
            continue
        
        chain2 = pairs2[key1]
        reg_start = chain1.getRowFrom()
        for pos in xrange(chain1.getRowFrom(), chain2.getRowTo()):
            x = chain1.mapRowToCol( pos )
            y = chain2.mapRowToCol( pos )

            if x == y: 
                if output_matches:
                    outfile.write( "%s\t%s\t%s\t%i\t%i\t%i\t%s\n" %
                                   ( key1 + (pos, x, y, "=") ) )
                continue

            mismatch =  x != -1 and y != -1
            
            if mismatch:
                if output_mismatches:
                    outfile.write( "%s\t%s\t%s\t%i\t%i\t%i\t%i\n" %
                                   ( key1 + (pos, x, y, x-y) ) )
            else:
                if output_unique:
                    outfile.write( "%s\t%s\t%s\t%i\t%i\t%i\t%s\n" %
                                   ( key1 + (pos, x, y, "-") ) )
                            

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: chain2psl.py 2899 2010-04-13 14:37:37Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option( "-m", "--output-mismatches", dest="output_mismatches", action = "store_true",
                       help = "output mismatches [%default]" )

    parser.add_option( "-a", "--output-matches", dest="output_matches", action = "store_true",
                       help = "output matches [%default]" )

    parser.add_option( "-u", "--output-unique", dest="output_unique", action = "store_true",
                       help = "output unique positions [%default]" )

    parser.add_option( "-r", "--restrict", dest="restrict", type = "string",
                       help = "restrict analysis to a chromosome pair (chr1:chr1:+) [%default]" )

    parser.set_defaults(
        output_mismatches = False,
        output_unique = False,
        restrict = None 
        )


    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    if len(args) != 2:
        raise ValueError( "expected two chain files" )

    filename_chain1, filename_chain2 = args

    E.info( "validating chain 1")
    if not validateChain( IOTools.openFile( filename_chain1 ) ):
        E.warn( "validation failed - exiting" )
        return 1
        
    E.info( "validating chain 2")
    if not validateChain( IOTools.openFile( filename_chain2 ) ):
        E.warn( "validation failed - exiting" )
        return 1

    E.info( "building pairs for %s" % filename_chain1 )
    pairs1 = buildPairs( IOTools.openFile( filename_chain1 ) )
    E.info( "read %i pairs" % len(pairs1) )

    E.info( "building pairs for %s" % filename_chain2 )
    pairs2 = buildPairs( IOTools.openFile( filename_chain2 ) )
    E.info( "read %i pairs" % len(pairs2) )

    if options.restrict: 
        restrict = tuple(options.restrict.split(":"))
        pairs1 = { restrict: pairs1[restrict] }
        pairs2 = { restrict: pairs2[restrict] }

    E.info( "comparing 1 -> 2")
    comparison1 = compareChains( pairs1, pairs2 )
    E.info( "comparing 2 -> 1")
    comparison2 = compareChains( pairs2, pairs1 )

    all_keys = sorted(list( set(comparison1.keys() + comparison2.keys())))
    
    outfile = options.stdout
    headers = ("mapped", "identical", "different", "unique")
    outfile.write( "contig1\tcontig2\tstrand\t%s\t%s\t%s\t%s\n" %\
                       ( 
            "\t".join( ["%s1" % x for x in headers ] ),
            "\t".join( ["p%s1" % x for x in headers ] ),
            "\t".join( ["%s2" % x for x in headers ] ),
            "\t".join( ["p%s2" % x for x in headers ] )))
                         
    totals = E.Counter()

    for key in all_keys:
        outfile.write( "%s\t%s\t%s" % key )
        
        if key in comparison1:
            c = comparison1[key]
            outfile.write( "\t%i\t%i\t%i\t%i\t" % (c.total, c.same, c.different, c.unique ) )
            outfile.write( "\t".join( [ IOTools.prettyPercent( x, c.total ) for x in c ] ) )

            totals.total1 += c.total
            totals.same1 += c.same
            totals.different1 += c.different
            totals.unique1 += c.unique
        else:
            outfile.write( "\t%i\t%i\t%i\t%i\t" % (0,0,0,0) )
            outfile.write( "\t%i\t%i\t%i\t%i" % (0,0,0,0) )

        if key in comparison2:
            c = comparison2[key] 
            outfile.write( "\t%i\t%i\t%i\t%i\t" % (c.total, c.same, c.different, c.unique ) )
            outfile.write( "\t".join( [ IOTools.prettyPercent( x, c.total ) for x in c ] ) )

            totals.same2 += c.same
            totals.total2 += c.total
            totals.different2 += c.different
            totals.unique2 += c.unique 
        else:
            outfile.write( "\t%i\t%i\t%i\t%i\t" % (0,0,0,0) )
            outfile.write( "\t%i\t%i\t%i\t%i" % (0,0,0,0) )

        outfile.write("\n")

    outfile.write( "total\ttotal\t.\t" )
    outfile.write( "\t".join( map(str, ( totals.total1,
                                         totals.same1, 
                                         totals.different1,
                                         totals.unique1,
                                         IOTools.prettyPercent( totals.total1, totals.total1 ),
                                         IOTools.prettyPercent( totals.same1, totals.total1 ),
                                         IOTools.prettyPercent( totals.different1, totals.total1 ),
                                         IOTools.prettyPercent( totals.unique1, totals.total1 ),
                                         totals.total2,
                                         totals.same2, 
                                         totals.different2,
                                         totals.unique2,
                                         IOTools.prettyPercent( totals.total2, totals.total2 ),
                                         IOTools.prettyPercent( totals.same2, totals.total2 ),
                                         IOTools.prettyPercent( totals.different2, totals.total2 ),
                                         IOTools.prettyPercent( totals.unique2, totals.total2 ),
                                         ) ) ) + "\n" )
                                        
    
    # output mismapped residues
    if options.output_mismatches or options.output_unique:
        outputMismatches( pairs1, pairs2,
                          output_mismatches = options.output_mismatches,
                          output_unique = options.output_unique,
                          output_matches = options.output_matches,
                          )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



'''
cgat.py - Computational Genomics Analysis Tools
===============================================

:Author: Andreas Heger, CGAT
:Release: $Id$
:Date: |today|
:Tags: Genomics

To use a specific tool, type::

    cgat <tool> [tool options] [tool arguments]

Tools are grouped by keywords. For this message and a list of
available keywords type::
    
    cgat --help

For a list of tools matching a certain keyword, type::

   cgat --help <keyword>

or::

   cgat --help all

for a list of all available tools.

To get help for a specific tool, type::

    cgat <tool> --help
'''

import os
import sys
import glob
import imp
import collections

def mapKeyword2Script( path ):
    '''collect keywords from scripts.'''
    
    map_keyword2script = collections.defaultdict( list )

    for script in glob.glob( os.path.join( path, "*.py") ):
        s = os.path.basename(script)[:-3]
        with open( script, 'r') as inf:
            data = [ x for x in inf.readlines( 10000 ) if x.startswith(':Tags:') ]
            if data:
                keywords = [x.strip() for x in data[0][6:].split(' ') ]
                for x in keywords:
                    if x: map_keyword2script[x].append( s )

    return map_keyword2script

def printListInColumns( l, ncolumns ):
    '''output list *l* in *ncolumns*.'''
    ll = len(l)

    if ll == 0: return

    max_width = max( [len(x) for x in l ] ) + 3
    n = ll // ncolumns
    if ll % 3 != 0: n += 1
        
    # build columns
    columns = [ l[x*n:x*n+n] for x in range(ncolumns ) ]

    # add empty fields for missing columns in last row
    for x in range( ncolumns - (len(l) % ncolumns)): columns[-(x+1)].append('')

    # convert to rows
    rows = zip( *columns )

    # build pattern for a row
    p = '%-' + str(max_width) + 's' 
    pattern = ' '.join( [ p for x in range(ncolumns) ] )
    
    # put it all together
    return '\n'.join( [ pattern % row for row in rows ] )

def main(argv = None):

    argv = sys.argv

    path = os.path.abspath( os.path.dirname(__file__) )
 
    if len(argv) == 1 or argv[1] == "--help" or argv[1] == "-h":
        print(globals()["__doc__"])

        map_keyword2script = mapKeyword2Script( path )

        if len(argv) <= 2:
            
            print('CGAT tools are grouped by keywords. The following keywords')
            print('are defined:\n')
            print("%s\n" % printListInColumns( map_keyword2script.keys(),
                                               3 ))

        if 'all' in argv[2:]:
            print("The list of all available commands is:\n" )
            print("%s\n" % printListInColumns( 
                    sorted([os.path.basename(x)[:-3] \
                                for x in glob.glob( os.path.join( path, "*.py") )]),
                    3 ))

        else:
            for arg in argv[2:]:
                if arg in map_keyword2script:
                    print ( "Tools matching the keyword '%s':\n" % arg )
                    print ( '%s\n' % printListInColumns(
                                        sorted(map_keyword2script[arg]),
                                        3 ) )
        return

    command = argv[1]

    (file, pathname, description ) = imp.find_module( command, [path,] )
    module = imp.load_module( command, file, pathname, description)
    module.main( sys.argv )
    
if __name__ == "__main__":
    sys.exit( main() )
